@@ -22,8 +22,8 @@ func BaseWithChallengeAndOGTags(title string, body templ.Component, impressum *c
 	}, ogTags, localizer)
 }
 
-func ErrorPage(msg, mail string, localizer *localization.SimpleLocalizer) templ.Component {
-	return errorPage(msg, mail, localizer)
+func ErrorPage(msg, mail, code string, localizer *localization.SimpleLocalizer) templ.Component {
+	return errorPage(msg, mail, code, localizer)
 }
 
 func Bench(localizer *localization.SimpleLocalizer) templ.Component {
